==> ponyc-0.21.3-2-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/collections-persistent-Vec.md <==
# Vec\[A: [Any](builtin-Any) #share\]

A persistent vector based on the Hash Array Mapped Trie from 'Ideal Hash
Trees' by Phil Bagwell.


```pony
class val Vec[A: Any #share]
```

## Constructors

### create

```pony
new val create()
: Vec[A] val^
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val^

---

### _create

```pony
new val _create(
  root': (_VecNode[A] val | None val),
  tail': Array[A] val,
  size': USize val,
  depth': U8 val,
  tail_offset': USize val)
: Vec[A] val^
```
#### Parameters

*   root': ([_VecNode](collections-persistent-_VecNode)\[A\] val | [None](builtin-None) val)
*   tail': [Array](builtin-Array)\[A\] val
*   size': [USize](builtin-USize) val
*   depth': [U8](builtin-U8) val
*   tail_offset': [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val^

---

## Public Functions

### size

Return the amount of values in the vector.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### apply

Get the i-th element, raising an error if the index is out of bounds.


```pony
fun box apply(
  i: USize val)
: A ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* A ?

---

### update

Return a vector with the i-th element changed, raising an error if the
index is out of bounds.


```pony
fun val update(
  i: USize val,
  value: A)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### insert

Return a vector with an element inserted. Elements after this are moved
up by one index, extending the vector. An out of bounds index raises an
error.


```pony
fun val insert(
  i: USize val,
  value: A)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### delete

Return a vector with an element deleted. Elements after this are moved
down by one index, compacting the vector. An out of bounds index raises an
error.


```pony
fun val delete(
  i: USize val)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### remove

Return a vector with n elements removed, beginning at index i.


```pony
fun val remove(
  i: USize val,
  n: USize val)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   n: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### push

Return a vector with the value added to the end.


```pony
fun val push(
  value: A)
: Vec[A] val
```
#### Parameters

*   value: A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### pop

Return a vector with the value at the end removed.


```pony
fun val pop()
: Vec[A] val ?
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### concat

Return a vector with the values of the given iterator added to the end.


```pony
fun val concat(
  iter: Iterator[A] ref)
: Vec[A] val
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[A\] ref

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### find

Find the `nth` appearance of `value` from the beginning of the vector,
starting at `offset` and examining higher indices, and using the
supplied `predicate` for comparisons. Returns the index of the value, or
raise an error if the value isn't present.

By default, the search starts at the first element of the vector,
returns the first instance of `value` found, and uses object identity
for comparison.


```pony
fun val find(
  value: A,
  offset: USize val = seq,
  nth: USize val = seq,
  predicate: {(A, A): Bool}[A] val = seq)
: USize val ?
```
#### Parameters

*   value: A
*   offset: [USize](builtin-USize) val = seq
*   nth: [USize](builtin-USize) val = seq
*   predicate: {(A, A): Bool}[A] val = seq

#### Returns

* [USize](builtin-USize) val ?

---

### contains

Returns true if the vector contains `value`, false otherwise.


```pony
fun val contains(
  value: A,
  predicate: {(A, A): Bool}[A] val = seq)
: Bool val
```
#### Parameters

*   value: A
*   predicate: {(A, A): Bool}[A] val = seq

#### Returns

* [Bool](builtin-Bool) val

---

### slice

Return a vector that is a clone of a portion of this vector. The range is
exclusive and saturated.


```pony
fun val slice(
  from: USize val = seq,
  to: USize val = seq,
  step: USize val = seq)
: Vec[A] val
```
#### Parameters

*   from: [USize](builtin-USize) val = seq
*   to: [USize](builtin-USize) val = seq
*   step: [USize](builtin-USize) val = seq

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### reverse

Return a vector with the elements in reverse order.


```pony
fun val reverse()
: Vec[A] val
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### keys

Return an iterator over the indices in the vector.


```pony
fun val keys()
: VecKeys[A] ref^
```

#### Returns

* [VecKeys](collections-persistent-VecKeys)\[A\] ref^

---

### values

Return an iterator over the values in the vector.


```pony
fun val values()
: VecValues[A] ref^
```

#### Returns

* [VecValues](collections-persistent-VecValues)\[A\] ref^

---

### pairs

Return an iterator over the (index, value) pairs in the vector.


```pony
fun val pairs()
: VecPairs[A] ref^
```

#### Returns

* [VecPairs](collections-persistent-VecPairs)\[A\] ref^

---

## Private Functions

### _pow32

Raise 32 to the power of n.


```pony
fun box _pow32(
  n: USize val)
: USize val
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [USize](builtin-USize) val

---

### _leaf_nodes

```pony
fun box _leaf_nodes()
: Array[Array[A] val] ref^
```

#### Returns

* [Array](builtin-Array)\[[Array](builtin-Array)\[A\] val\] ref^

---

==> ponyc-0.23.0-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/collections-persistent-Vec.md <==
# Vec\[A: [Any](builtin-Any) #share\]
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L3)</span>

A persistent vector based on the Hash Array Mapped Trie from 'Ideal Hash
Trees' by Phil Bagwell.


```pony
class val Vec[A: Any #share]
```

## Constructors

### create
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L14)</span>


```pony
new val create()
: Vec[A] val^
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val^

---

### _create
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L20)</span>


```pony
new val _create(
  root': (_VecNode[A] val | None val),
  tail': Array[A] val,
  size': USize val,
  depth': USize val)
: Vec[A] val^
```
#### Parameters

*   root': ([_VecNode](collections-persistent-_VecNode)\[A\] val | [None](builtin-None) val)
*   tail': [Array](builtin-Array)\[A\] val
*   size': [USize](builtin-USize) val
*   depth': [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val^

---

## Public Functions

### size
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L31)</span>


Return the amount of values in the vector.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### apply
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L43)</span>


Get the i-th element, raising an error if the index is out of bounds.


```pony
fun box apply(
  i: USize val)
: val->A ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* val->A ?

---

### update
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L53)</span>


Return a vector with the i-th element changed, raising an error if the
index is out of bounds.


```pony
fun val update(
  i: USize val,
  value: val->A)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: val->A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### insert
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L67)</span>


Return a vector with an element inserted. Elements after this are moved
up by one index, extending the vector. An out of bounds index raises an
error.


```pony
fun val insert(
  i: USize val,
  value: val->A)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: val->A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### delete
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L81)</span>


Return a vector with an element deleted. Elements after this are moved
down by one index, compacting the vector. An out of bounds index raises an
error.


```pony
fun val delete(
  i: USize val)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### remove
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L94)</span>


Return a vector with n elements removed, beginning at index i.


```pony
fun val remove(
  i: USize val,
  n: USize val)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   n: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### push
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L106)</span>


Return a vector with the value added to the end.


```pony
fun val push(
  value: val->A)
: Vec[A] val
```
#### Parameters

*   value: val->A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### pop
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L138)</span>


Return a vector with the value at the end removed.


```pony
fun val pop()
: Vec[A] val ?
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### concat
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L156)</span>


Return a vector with the values of the given iterator added to the end.


```pony
fun val concat(
  iter: Iterator[val->A] ref)
: Vec[A] val
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[val->A\] ref

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### find
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L166)</span>


Find the `nth` appearance of `value` from the beginning of the vector,
starting at `offset` and examining higher indices, and using the
supplied `predicate` for comparisons. Returns the index of the value, or
raise an error if the value isn't present.

By default, the search starts at the first element of the vector,
returns the first instance of `value` found, and uses object identity
for comparison.


```pony
fun val find(
  value: val->A,
  offset: USize val = seq,
  nth: USize val = seq,
  predicate: {(A, A): Bool}[A] val = seq)
: USize val ?
```
#### Parameters

*   value: val->A
*   offset: [USize](builtin-USize) val = seq
*   nth: [USize](builtin-USize) val = seq
*   predicate: {(A, A): Bool}[A] val = seq

#### Returns

* [USize](builtin-USize) val ?

---

### contains
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L192)</span>


Returns true if the vector contains `value`, false otherwise.


```pony
fun val contains(
  value: val->A,
  predicate: {(A, A): Bool}[A] val = seq)
: Bool val
```
#### Parameters

*   value: val->A
*   predicate: {(A, A): Bool}[A] val = seq

#### Returns

* [Bool](builtin-Bool) val

---

### slice
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L205)</span>


Return a vector that is a clone of a portion of this vector. The range is
exclusive and saturated.


```pony
fun val slice(
  from: USize val = seq,
  to: USize val = seq,
  step: USize val = seq)
: Vec[A] val
```
#### Parameters

*   from: [USize](builtin-USize) val = seq
*   to: [USize](builtin-USize) val = seq
*   step: [USize](builtin-USize) val = seq

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### reverse
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L216)</span>


Return a vector with the elements in reverse order.


```pony
fun val reverse()
: Vec[A] val
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### keys
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L226)</span>


Return an iterator over the indices in the vector.


```pony
fun val keys()
: VecKeys[A] ref^
```

#### Returns

* [VecKeys](collections-persistent-VecKeys)\[A\] ref^

---

### values
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L232)</span>


Return an iterator over the values in the vector.


```pony
fun val values()
: VecValues[A] ref^
```

#### Returns

* [VecValues](collections-persistent-VecValues)\[A\] ref^

---

### pairs
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L238)</span>


Return an iterator over the (index, value) pairs in the vector.


```pony
fun val pairs()
: VecPairs[A] ref^
```

#### Returns

* [VecPairs](collections-persistent-VecPairs)\[A\] ref^

---

## Private Functions

### _tail_offset
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L37)</span>


Return the amount of values in the root.


```pony
fun box _tail_offset()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### _pow32
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L244)</span>


Raise 32 to the power of n.


```pony
fun box _pow32(
  n: USize val)
: USize val
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [USize](builtin-USize) val

---

### _leaf_nodes
<span class="source-link">[[Source]](src/collections-persistent/vec.md#L254)</span>


```pony
fun box _leaf_nodes()
: Array[Array[A] val] ref^
```

#### Returns

* [Array](builtin-Array)\[[Array](builtin-Array)\[A\] val\] ref^

---
